#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// The Middle High Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
    /// </summary>
    [Description("The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities ")]
    public class MHL : Indicator
    {
        #region Variables
		private int maPeriod = 10;
        private int mhlPeriod = 3;
		private DataSeries mhl;
		private SelectableMA	matype	= SelectableMA.SMA;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "MHLA"));
            Overlay				= true;
			
			mhl = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            double HH = MAX(High, mhlPeriod)[0];
			double LL = MIN(Low, mhlPeriod)[0];
			mhl[0] = (HH+LL)/2;
			
            Values[0][0] = SMA(mhl, maPeriod)[0];
			
			switch (matype)
			{
				case SelectableMA.EMA:
				{
					Values[0][0] = EMA(mhl, maPeriod)[0];
					break;
				}
				case SelectableMA.SMA:
				{
					Values[0][0] = SMA(mhl, maPeriod)[0];
					break;
				}
			}
        }

        #region Properties
		[Description("Choose a Moving Average type.")]
		[GridCategory("Parameters")]
		public SelectableMA MaType
		{
			get { return matype; }
			set { matype = value; }
		}
		
        [Description("Bars to calculate Middle-High-Low")]
        [GridCategory("Parameters")]
        public int MhlPeriod
        {
            get { return mhlPeriod; }
            set { mhlPeriod = Math.Max(1, value); }
        }
		
		[Description("Bars to calculate Moving Average")]
        [GridCategory("Parameters")]
        public int MaPeriod
        {
            get { return maPeriod; }
            set { maPeriod = Math.Max(1, value); }
        }
        #endregion
    }
}

public enum SelectableMA
{
	EMA,
	SMA,
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MHL[] cacheMHL = null;

        private static MHL checkMHL = new MHL();

        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        public MHL MHL(int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            return MHL(Input, maPeriod, maType, mhlPeriod);
        }

        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        public MHL MHL(Data.IDataSeries input, int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            if (cacheMHL != null)
                for (int idx = 0; idx < cacheMHL.Length; idx++)
                    if (cacheMHL[idx].MaPeriod == maPeriod && cacheMHL[idx].MaType == maType && cacheMHL[idx].MhlPeriod == mhlPeriod && cacheMHL[idx].EqualsInput(input))
                        return cacheMHL[idx];

            lock (checkMHL)
            {
                checkMHL.MaPeriod = maPeriod;
                maPeriod = checkMHL.MaPeriod;
                checkMHL.MaType = maType;
                maType = checkMHL.MaType;
                checkMHL.MhlPeriod = mhlPeriod;
                mhlPeriod = checkMHL.MhlPeriod;

                if (cacheMHL != null)
                    for (int idx = 0; idx < cacheMHL.Length; idx++)
                        if (cacheMHL[idx].MaPeriod == maPeriod && cacheMHL[idx].MaType == maType && cacheMHL[idx].MhlPeriod == mhlPeriod && cacheMHL[idx].EqualsInput(input))
                            return cacheMHL[idx];

                MHL indicator = new MHL();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.MaPeriod = maPeriod;
                indicator.MaType = maType;
                indicator.MhlPeriod = mhlPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                MHL[] tmp = new MHL[cacheMHL == null ? 1 : cacheMHL.Length + 1];
                if (cacheMHL != null)
                    cacheMHL.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheMHL = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MHL MHL(int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            return _indicator.MHL(Input, maPeriod, maType, mhlPeriod);
        }

        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        public Indicator.MHL MHL(Data.IDataSeries input, int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            return _indicator.MHL(input, maPeriod, maType, mhlPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MHL MHL(int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            return _indicator.MHL(Input, maPeriod, maType, mhlPeriod);
        }

        /// <summary>
        /// The Middle-High-Low Moving Average as described in August 2015 Technical Analysis Stocks and Commodities 
        /// </summary>
        /// <returns></returns>
        public Indicator.MHL MHL(Data.IDataSeries input, int maPeriod, SelectableMA maType, int mhlPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MHL(input, maPeriod, maType, mhlPeriod);
        }
    }
}
#endregion
